import pandas as pd
from scipy.stats import entropy

from CalculatingFeatures.helper_functions import convertInputInto2dTime, convert2DEmpaticaToArray hrvFeatureNames, hrvFreqFeatureNames
from CalculatingFeatures.calculate_features import calculateFeatures

import sys


def getSampleRate(data):
    try:
        timestamps_diff = data['timestamp'].diff().dropna().mean()
    except:
        raise Exception("Error occured while trying to get the mean sample rate from the data.")

    return 1000/timestamps_diff

def extractIBIFeaturesFromIntradayData(ibi_intraday_data, features, window_length, time_segment, filter_data_by_segment):
    ibi_intraday_features = pd.DataFrame(columns=["local_segment"] + features)

    if not ibi_intraday_data.empty:   
        sample_rate = getSampleRate(ibi_intraday_data)   
     
        ibi_intraday_data = filter_data_by_segment(ibi_intraday_data, time_segment)

        if not ibi_intraday_data.empty:

            ibi_intraday_features = pd.DataFrame()

            # apply methods from calculate features module
            # if window_length is None:
            #     ibi_intraday_features = \
            #         ibi_intraday_data.groupby('local_segment').apply(\
            #         lambda x: calculateFeatures(convertInputInto2d(x['blood_volume_pulse'], x.shape[0]), fs=int(sample_rate), featureNames=features)) 
            # else:
            #     ibi_intraday_features = \
            #         ibi_intraday_data.groupby('local_segment').apply(\
            #         lambda x: calculateFeatures(convertInputInto2d(x['blood_volume_pulse'], window_length*int(sample_rate)), fs=int(sample_rate), featureNames=features)) 

            ibi_intraday_features.reset_index(inplace=True)

    return ibi_intraday_features


def cr_features(sensor_data_files, time_segment, provider, filter_data_by_segment, *args, **kwargs):
    ibi_intraday_data = pd.read_csv(sensor_data_files["sensor_data"])

    requested_intraday_features = provider["FEATURES"]
    
    calc_windows = kwargs.get('calc_windows', False)

    if provider["WINDOWS"]["COMPUTE"] and calc_windows:
        requested_window_length = provider["WINDOWS"]["WINDOW_LENGTH"]
    else:
        requested_window_length = None

    # name of the features this function can compute
    base_intraday_features_names = hrvFeatureNames + hrvFreqFeatureNames
    # the subset of requested features this function can compute
    intraday_features_to_compute = list(set(requested_intraday_features) & set(base_intraday_features_names))

    # extract features from intraday data
    ibi_intraday_features = extractBVPFeaturesFromIntradayData(ibi_intraday_data, intraday_features_to_compute, 
                                                                requested_window_length, time_segment, filter_data_by_segment)

    return ibi_intraday_features